@@ -30,7 +30,6 @@ class MessageModel(ABC):
             A dictionary with "format" and "content" keys.
 
         """
-
         return {"format": type(self).__name__.lower(), "content": self.asdict()}
 
     @classmethod
@@ -51,6 +50,7 @@ class MessageModel(ABC):
     @abstractmethod
     def load(cls, input_):
         """Create a new message model from a file object or string.
+        This base implementation has no functionality and should not be called.
 
         Args:
             input_: A file object or string.
@@ -59,7 +59,7 @@ class MessageModel(ABC):
             The message model.
 
         """
-        pass
+        raise NotImplementedError("MessageModel.load() should not be called")
 
 
 @dataclass
@@ -182,6 +182,15 @@ class Blob(MessageModel):
     def __str__(self):
         return str(self.content)
 
+    def serialize(self):
+        """Wrap the message with its format and content.
+
+        Returns:
+            A dictionary with "format" and "content" keys.
+
+        """
+        return {"format": type(self).__name__.lower(), "content": self.content}
+
     def asdict(self):
         """Represents the message as a dictionary.
 
