@@ -104,6 +104,34 @@ def test_stream_read(circular_msg):
         assert messages == 1
 
 
+def test_stream_read_test_channel(circular_msg):
+    start_at = io.StartPosition.EARLIEST
+    message_data = {"format": "circular", "content": circular_msg}
+    fake_message = MagicMock()
+    fake_message.value = MagicMock(return_value=json.dumps(message_data).encode("utf-8"))
+
+    def test_headers():
+        return [('_test', b'true')]
+
+    fake_message.headers = test_headers
+    mock_instance = MagicMock()
+    mock_instance.stream = MagicMock(return_value=[fake_message])
+    stream = io.Stream(start_at=start_at, until_eos=True, auth=False)
+    with patch("hop.io.consumer.Consumer", MagicMock(return_value=mock_instance)):
+        broker_url = "kafka://hostname:port/test-topic"
+        messages = 0
+        with stream.open(broker_url, "r") as s:
+            for msg in s:
+                messages += 1
+        assert messages == 0
+
+        messages = 0
+        with stream.open(broker_url, "r", ignoretest=False) as s:
+            for msg in s:
+                messages += 1
+        assert messages == 1
+
+
 def test_stream_read_multiple(circular_msg):
     group_id = "test-group"
     topic1 = "gcn1"
@@ -136,7 +164,14 @@ def test_stream_write(circular_msg, circular_text, mock_broker, mock_producer):
     topic = "gcn"
     mock_adc_producer = mock_producer(mock_broker, topic)
     expected_msg = json.dumps(Blob(circular_msg).serialize()).encode("utf-8")
-    headers = {"some header": "some value"}
+
+    headers = {"some header": "some value", "another header": b"other value"}
+    canonical_headers = [("some header", "some value"), ("another header", b"other value")]
+    test_headers = canonical_headers.copy()
+    test_headers.append(('_test', b'true'))
+    none_test_headers = []
+    none_test_headers.append(('_test', b"true"))
+
     with patch("hop.io.producer.Producer", autospec=True, return_value=mock_adc_producer):
 
         broker_url = f"kafka://localhost:port/{topic}"
@@ -157,14 +192,48 @@ def test_stream_write(circular_msg, circular_text, mock_broker, mock_producer):
         mock_broker.reset()
         with stream.open(broker_url, "w") as s:
             s.write(circular_msg, headers)
-            assert mock_broker.has_message(topic, expected_msg, headers)
+            assert mock_broker.has_message(topic, expected_msg, canonical_headers)
+
+        mock_broker.reset()
+        with stream.open(broker_url, "w") as s:
+            s.write(circular_msg, headers, test=True)
+            assert mock_broker.has_message(topic, expected_msg, test_headers)
+
+        mock_broker.reset()
+        with stream.open(broker_url, "w") as s:
+            s.write(circular_msg, headers=None, test=True)
+            assert mock_broker.has_message(topic, expected_msg, none_test_headers)
 
         # repeat, but with a manual close instead of context management
         mock_broker.reset()
         s = stream.open(broker_url, "w")
         s.write(circular_msg, headers)
         s.close()
-        assert mock_broker.has_message(topic, expected_msg, headers)
+        assert mock_broker.has_message(topic, expected_msg, canonical_headers)
+
+
+def test_stream_write_raw(circular_msg, circular_text, mock_broker, mock_producer):
+    topic = "gcn"
+    mock_adc_producer = mock_producer(mock_broker, topic)
+    encoded_msg = io.Producer.pack(GCNCircular(circular_msg["header"], circular_msg["body"]))
+    headers = {"some header": "some value"}
+    canonical_headers = list(headers.items())
+    with patch("hop.io.producer.Producer", autospec=True, return_value=mock_adc_producer):
+        stream = io.Stream(auth=False)
+
+        broker_url = f"kafka://localhost:9092/{topic}"
+
+        mock_broker.reset()
+        with stream.open(broker_url, "w") as s:
+            s.write_raw(encoded_msg, canonical_headers)
+            assert mock_broker.has_message(topic, encoded_msg, canonical_headers)
+
+        # repeat, but with a manual close instead of context management
+        mock_broker.reset()
+        s = stream.open(broker_url, "w")
+        s.write_raw(encoded_msg, canonical_headers)
+        s.close()
+        assert mock_broker.has_message(topic, encoded_msg, canonical_headers)
 
 
 def test_stream_auth(auth_config, tmpdir):
@@ -261,11 +330,11 @@ def test_mark_done(circular_msg):
 def test_pack(circular_msg, circular_text):
     # message class
     circular = GCNCircular(**circular_msg)
-    packed_msg = io.Producer._pack(circular)
+    packed_msg, _ = io.Producer.pack(circular)
 
     # unstructured message
     message = {"hey": "you"}
-    packed = io.Producer._pack(message)
+    packed, _ = io.Producer.pack(message)
 
 
 @pytest.mark.parametrize("message", [
@@ -290,7 +359,7 @@ def test_pack_unpack_roundtrip(message, message_parameters_dict, caplog):
         orig_message = test_file.read_text()
 
     # pack the message
-    packed_msg = io.Producer._pack(orig_message)
+    packed_msg, _ = io.Producer.pack(orig_message)
 
     # mock a kafka message with value being the packed message
     kafka_msg = MagicMock()
@@ -312,7 +381,7 @@ def test_pack_unpack_roundtrip_unstructured():
     # objects (of types that json.loads happens to produce) should remain unchanged by the process
     # of packing and unpacking
     for orig_message in ["a string", ["a", "B", "c"], {"dict": True, "other_data": [5, 17]}]:
-        packed_msg = io.Producer._pack(orig_message)
+        packed_msg, _ = io.Producer.pack(orig_message)
         kafka_msg = MagicMock()
         kafka_msg.value.return_value = packed_msg
         unpacked_msg = io.Consumer._unpack(kafka_msg)
@@ -321,7 +390,7 @@ def test_pack_unpack_roundtrip_unstructured():
     # non-serializable objects should raise an error
     with pytest.raises(TypeError):
         # note that we are not trying to pack a string, but the string class itself
-        packed_msg = io.Producer._pack(str)
+        packed_msg, _ = io.Producer.pack(str)
 
 
 @pytest.mark.parametrize("message", [
@@ -418,6 +487,38 @@ def test_plugin_loading(caplog):
         assert "BLOB" in registered
 
 
+def test_stream_flush():
+    # flush is pretty trivial; it should just call the underlying flush
+    with patch("adc.producer.Producer.flush", MagicMock()) as flush:
+        broker_url = "kafka://example.com:9092/topic"
+        stream = io.Stream(auth=False).open(broker_url, "w")
+        stream.flush()
+        flush.assert_called()
+
+
+def test_is_test(circular_msg):
+    fake_message = MagicMock()
+
+    def fake_headers_none():
+        return None
+
+    def fake_headers_list_test():
+        return [('_test', b'true')]
+
+    def fake_headers_list():
+        return [('foo', b'bar')]
+
+    fake_message.headers = fake_headers_none
+    ret = io.Consumer.is_test(fake_message)
+    assert ret is False
+    fake_message.headers = fake_headers_list_test
+    ret = io.Consumer.is_test(fake_message)
+    assert ret is True
+    fake_message.headers = fake_headers_list
+    ret = io.Consumer.is_test(fake_message)
+    assert ret is False
+
+
 def make_mock_listing_consumer(topics=[]):
     """Create a mock object suitable for replacing confluent_kafka.Consumer
         which has a list_topics method which acts as if a predetermined set of
